<?php
/**
 * PHP version 5.6+
 *
 * LICENSE: This source file is subject to version 3.01 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_01.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @author     Joseph G. <emailnotpublic@domain.tld>
 * @license    http://www.php.net/license/3_01.txt  PHP License 3.01
 * @version    0.0.4
 * @link       https://github.com/codehaiku/Envato-Purchase-Code-Verifier
 * @since      0.0.1
 */

require_once __DIR__ . '/vendor/autoload.php';

use Curl\Curl;

final class EnvatoPurchaseCodeVerifier
{
    protected $accessTokens = [];
    protected $debug = [];

    const AUTHOR_SALES_ENDPOINT_URI = 'https://api.envato.com/v3/market/author/sale';
    const DEFAULT_SUPPORT_PERIOD = 6; // Default support period in months if not specified in the API response

    /**
     * Class constructor, pass the access tokens.
     * @param mixed $accessTokens Single token string or array of tokens
     */
    public function __construct($accessTokens)
    {
        if (is_array($accessTokens)) {
            $this->accessTokens = $accessTokens;
        } else {
            $this->accessTokens = [$accessTokens];
        }
    }

    /**
     * Add a new access token
     * @param string $accessToken The access token to add
     */
    public function addAccessToken($accessToken)
    {
        if (!in_array($accessToken, $this->accessTokens)) {
            $this->accessTokens[] = $accessToken;
        }
    }

    /**
     * Get all access tokens
     * @return array Array of access tokens
     */
    public function getAccessTokens()
    {
        return $this->accessTokens;
    }

    /**
     * Get debug information
     * @return array Debug information
     */
    public function getDebugInfo()
    {
        return $this->debug;
    }

    /**
     * Calculate support end date based on purchase date and supported_until from API
     * @param string $purchaseDate Purchase date in format 'YYYY-MM-DD HH:MM:SS'
     * @param string|null $supportedUntil Support end date from API (if available)
     * @param int|null $supportPeriod Support period in months (if specified)
     * @return array Support information including end date and status
     */
    public function getSupportInfo($purchaseDate, $supportedUntil = null, $supportPeriod = null)
    {
        $purchase = new DateTime($purchaseDate);
        $now = new DateTime();

        // If we have supported_until date from API, use it
        if ($supportedUntil) {
            $supportEnd = new DateTime($supportedUntil);
        } else {
            // Otherwise calculate based on support period or default
            $supportEnd = clone $purchase;
            $months = $supportPeriod ?: self::DEFAULT_SUPPORT_PERIOD;
            $supportEnd->modify('+' . $months . ' months');
        }

        return [
            'purchase_date' => $purchase->format('Y-m-d H:i:s'),
            'support_end_date' => $supportEnd->format('Y-m-d H:i:s'),
            'support_active' => ($now < $supportEnd),
            'days_remaining' => max(0, $supportEnd->diff($now)->days),
            'support_expired' => ($now > $supportEnd),
            'support_length' => $purchase->diff($supportEnd)->m + ($purchase->diff($supportEnd)->y * 12)
        ];
    }
    
    /**
     * Supply the purchase code of the buyer as argument.
     * @param  mixed $purchaseCode The purchase code of the buyer.
     * @return mixed (Boolean) False on fail, array with verification and support info on success.
     */
    public function verified($purchaseCode)
    {
        if (empty($purchaseCode)) {
            $this->debug[] = ['error' => 'Purchase code is empty'];
            return false;
        }

        // Format check (Envato purchase codes are typically 36 characters)
        if (strlen($purchaseCode) !== 36) {
            $this->debug[] = ['error' => 'Invalid purchase code format'];
            return false;
        }

        $response = wp_remote_get(
            self::AUTHOR_SALES_ENDPOINT_URI . '?code=' . $purchaseCode,
            array(
                'headers' => array(
                    'Authorization' => 'Bearer ' . $this->accessTokens[0],
                    'User-Agent' => 'WordPress/' . get_bloginfo('version')
                ),
                'timeout' => 20
            )
        );

        // Сохраняем отладочную информацию о запросе
        $this->debug[] = [
            'request_url' => self::AUTHOR_SALES_ENDPOINT_URI . '?code=' . $purchaseCode,
            'response_code' => wp_remote_retrieve_response_code($response),
            'response_message' => wp_remote_retrieve_response_message($response),
            'is_wp_error' => is_wp_error($response)
        ];

        if (is_wp_error($response)) {
            $this->debug[] = ['error' => $response->get_error_message()];
            return false;
        }

        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code !== 200) {
            $this->debug[] = [
                'error' => 'Invalid response code: ' . $response_code,
                'response_body' => wp_remote_retrieve_body($response)
            ];
            return false;
        }

        $body = json_decode(wp_remote_retrieve_body($response));
        
        if (empty($body)) {
            $this->debug[] = ['error' => 'Empty response body'];
            return false;
        }

        // Добавляем тело ответа в отладочную информацию
        $this->debug[] = ['response_body' => $body];

        // Если есть информация о покупке, добавляем информацию о поддержке
        if (isset($body->sold_at)) {
            $body->support_info = $this->getSupportInfo(
                $body->sold_at,
                isset($body->supported_until) ? $body->supported_until : null,
                isset($body->support_amount) ? $body->support_amount : null
            );
        }

        return $body;
    }
}
